;------------------------- FIND_COEFFS -------------------------------------
function Find_coeffs,tvalue,S,R,d,vx,vy,vz,Ax,Ay,Az
vect_v=[-S(1),S(0),d/(2*!pi)] ;vektor rychlosti
vect_n=vect_v/(sqrt(S(1)^2+S(0)^2+d^2/(4*!pi*!pi))) ;normalovy vektor ve smeru v
coef_A=vect_n(0) ;rovnice roviny Ax+By+Cz+D=0
coef_B=vect_n(1)
coef_C=vect_n(2)
coef_D=-coef_A*S(0)-coef_B*S(1)-coef_C*S(2)
coeffs=dblarr(2)
coeffs(0)=double(coef_A*vx+coef_B*vy+coef_C*vz)
coeffs(1)=double(coef_A*Ax+coef_B*Ay+coef_C*Az+coef_D)
return,coeffs
end

;----------------------- NUMERICAL_SEARCH --------------------------
function Numerical_search,tlimit,distances,spiral,bisector,steps,Acc ;kvuli rekurzi
end

;hleda prusecik primky a spiraly v uzkem intervalu pulenim intervalu
function Numerical_search,tlimit,distances,spiral,bisector,steps,Acc
t_down=tlimit(0)
t_up=tlimit(1)
dist_down=distances(0)
dist_up=distances(1)
R=spiral(0)
d=spiral(1)
w=spiral(2)
vx=bisector(0)
vy=bisector(1)
vz=bisector(2)
Ax=bisector(3)
Ay=bisector(4)
Az=bisector(5)

t_mid=(t_up+t_down)/2.0
S=[R*cos(t_mid),R*sin(t_mid),t_mid*d/(2*!pi)] ;bod na spirale odpovidajici danemu t
coeffs=Find_coeffs(t_mid,S,R,d,vx,vy,vz,Ax,Ay,Az)
if (coeffs(0) eq 0.0) then message,'Zero coefficient encountered, choose another numerical step'
                         ;tento coef by nemel byt nulovy (znaci rovnobeznost primky s rovinou)
t=-coeffs(1)/coeffs(0)
x=Ax+t*vx
y=Ay+t*vy
z=Az+t*vz
dist_mid=sqrt((x-S(0))^2+(y-S(1))^2+(z-S(2))^2)-w ;vzdalenost pruseciku od bodu na spirale-w

if (abs(dist_mid) le Acc) then begin ;pokud nalezen prusecik tak vrat jeho souradnice
     ;print,'Finished, number of steps: ',steps
     return,[x,y,z,t_mid]
endif else begin ;pokud prusecik nenalezen, pokracuj rekurzi
     if (dist_down*dist_mid) gt 0.0 then begin
         return,Numerical_search([t_mid,t_up],[dist_mid,dist_up],[R,d,w],[vx,vy,vz,Ax,Ay,Az],steps+1,Acc) ;$
     endif else begin
         return,Numerical_search([t_down,t_mid],[dist_down,dist_mid],[R,d,w],[vx,vy,vz,Ax,Ay,Az],steps+1,Acc)
     endelse
endelse
end

;------------------------- GET_DOPPLER ----------------------------
;pocita dopplerovskou slozku rychlosti
;vstup: vektor rychlosti a 2 body urcujici smer k pozorovateli
function Get_doppler,velocity,point1,point2
observer=point1-point2 ;vektor smerem k pozorovateli
lenght=sqrt(total(observer^2))
observer=observer/lenght ;jednotkovy vektor ve smeru k pozorovateli
doppler=total(velocity*observer) ;skalarni soucin
return,doppler
end

;--------------------- GET_SPIRAL_VELOCITY--------------------------
;pocita vektor rychlosti v danem bode spiraly
function Get_spiral_velocity,tlevel,State
S=[State.R*cos(tlevel),State.R*sin(tlevel),tlevel*State.dist/(2*!pi)]
vect_v=[-S(1),S(0),State.dist/(2*!pi)] ;vektor rychlosti (pro omega=1.0)
vect_n=vect_v/(sqrt(S(1)^2+S(0)^2+State.dist^2/(4*!pi*!pi))) ;normalovy vektor ve smeru pohybu
VS=State.V*0.001
velocity=VS*vect_n
return,velocity
end

;------------------------- GET_VELOCITY ----------------------------
;pocita vektor rychlosti v danem bode objektu
function Get_velocity,point,State
x=point(0)
y=point(1)
z=point(2)
Rad=sqrt(x^2+y^2) ;vzdalenost bodu od rotacni osy
case State.ActualObject of
       'cone':begin
              if z eq 0.0 then Radius=0.0 else Radius=Rad*State.Height/z ;kuzel s parametry State.Height a Radius  ;horni polomer pro dany bod
              vrot=State.ParamRot/100.0*Rad/(State.dTime) ;velikost rotacni rychlosti
              vp=State.ParamTex*0.001
              sintheta=Radius/sqrt(State.Height^2+Radius^2)
              costheta=State.Height/sqrt(State.Height^2+Radius^2)
              end
   'ellipsoid':begin
              a=State.a
              b=Rad/sqrt(1-(1-z/a)^2) ;b pro dane a a dany bod
              vrot=State.ParamRot/100.0*Rad/(State.dTime) ;velikost rotacni rychlosti
              vp=State.ParamTex*0.001
              if (z eq a) or (Rad eq 0.0) then begin ;na rotacni ose krom bodu z=0 rychlost pouze nahoru
                 costheta=1.0 & sintheta=0.0
              endif
              if (z eq 0.0) then begin ;pro bod z=0 zadna rychlost nahoru (pro tento bod zaroven plati Rad=0)
                 costheta=0.0 & sintheta=1.0
              endif
              if ((z ne a) and (Rad ne 0.0) and (z ne 0.0)) then begin
                   u=sqrt(b^4/Rad^2+a^4/(a-z)^2)
                   costheta=(a^2/(a-z))/u
                   sintheta=(b^2/Rad)/u
              endif
              end
   'cylinder':begin
              vrot=State.ParamRot/100.0*Rad/(State.dTime) ;velikost rotacni rychlosti
              vp=State.ParamTex*0.001
              sintheta=0.0
              costheta=1.0
              end

endcase

if Rad ne 0.0 then begin
   sinfi=y/Rad
   cosfi=x/Rad
   velocity1=[-vrot*sinfi,vrot*cosfi,0] ;vektor rotacni rychlosti
   velocity2=[vp*sintheta*cosfi,vp*sintheta*sinfi,vp*costheta] ;vektor posuvne rychlosti
endif else begin ;bod na rotacni ose
   velocity1=[0.0,0.0,0.0]
   velocity2=[0.0,0.0,vp]
endelse

return,velocity1+velocity2
end

;------------------------ GET_PLANE_POINTS --------------------------
;pocita pruseciky primky s rovinou
;point je bod roviny, bisector jsou koef. primky, rovina je kolma na osu z
function Get_plane_points,point,bisector,limit=limit
vx=bisector(0)
vy=bisector(1)
vz=bisector(2)
Ax=bisector(3)
Ay=bisector(4)
Az=bisector(5)
coef_A=0.0 ;rovnice roviny Ax+By+Cz+D=0
coef_B=0.0
coef_C=1.0
coef_D=-point(2)
if (vz eq 0.0) then return,-1 else begin ;pokud existuje prave jeden prusecik
   t=-double(Az+coef_D)/double(vz)
   x=Ax+t*vx
   y=Ay+t*vy
   z=Az+t*vz
   if (sqrt(x^2+y^2) lt limit(0)) and (sqrt(x^2+y^2) ge limit(1)) then return,[x,y,z] else return,-1
endelse
end

;------------------------ GET_INNER_POINTS --------------------------
;pocita pruseciky primky s objektem
function Get_inner_points,coeff,bisector,limit=limit
vx=bisector(0)
vy=bisector(1)
vz=bisector(2)
Ax=bisector(3)
Ay=bisector(4)
Az=bisector(5)
t=fz_roots(coeff,/double) ;resi kvadratickou rovnici pro t

if (imaginary(t(0)) ne 0.0) then return,-1 ;zadne realne reseni

t=double(t)
x=Ax+t*vx
y=Ay+t*vy
z=Az+t*vz
index=where(((z le limit) and (z ge 0.0)),count) ;souradnice z musi byt v objektu

if (count eq 0) then return,-1 ;pruseciky jsou mimo rozsah

x=x(index)
y=y(index)
z=z(index)

if ((count eq 2) and (t(0) eq t(1))) then return,[x(0),y(0),z(0)] else return,transpose([[x],[y],[z]])
end

;----------------------- MAKE_COUPLE ------------------------------
function Make_couple,p1,p2
   if (n_elements(p1) eq 1) then return,p2
   if (n_elements(p2) eq 1) then return,p1
   return,[[p1],[p2]] ;mely by byt max dva pruseciky
end

;----------------------- MAKE_COUPLES ------------------------------
function Make_couples,p1,p2,p3
n1=n_elements(p1)
n2=n_elements(p2)
n3=n_elements(p3)
if (n1 eq 1) and (n2 eq 1) and (n3 eq 1) then return,-1
if (n1 eq 6) and (n2 eq 1) and (n3 eq 1) then return,p1
if (n1 eq 1) and (n2 eq 1) and (n3 eq 6) then return,p3
if (n1 eq 6) and (n2 eq 6) and (n3 eq 1) then return,[[p1(*,0)],[p2(*,0)],[p2(*,1)],[p1(*,1)]]
if (n1 eq 3) and (n2 eq 1) and (n3 eq 3) then return,[[p1],[p3]]
if (n1 eq 3) and (n2 eq 3) and (n3 eq 1) then return,[[p1],[p2]]
if (n1 eq 1) and (n2 eq 3) and (n3 eq 3) then return,[[p2],[p3]]

if (n2 eq 6) then begin
   if p2(2,0) gt p2(2,1) then begin
      gz=p2(*,0)
      lz=p2(*,1)
   endif else begin
      gz=p2(*,1)
      lz=p2(*,0)
   endelse
   if (n1 eq 3) and (n3 eq 3) then begin
      if p1(2) lt p3(2) then begin
          lz2=p1 & gz2=p3
      endif else begin
          lz2=p3 & gz2=p1
      endelse
      return,[[lz2],[lz],[gz],[gz2]]
   endif
   if (n1 eq 1) and (n3 eq 6) then return,[[p3(*,1)],[lz],[gz],[p3(*,0)]]
endif
return,-1 ;mel by byt sudy pocet pruseciku
end

;------------------------- SIGNUM ----------------------------------
function Signum,arr
absarr=abs(arr)
index=where(arr ne 0.0)
sgnarr=arr
sgnarr(index)=arr(index)/absarr(index)
return,sgnarr
end

;----------------------- GET_SPIRAL_POINTS -------------------------
function Get_spiral_points,point1,point2,State
levels=State.Levels
accuracy=10.0d^State.Accuracy
R=State.R ;parametry spiraly
d=State.dist
h=State.Height
w=State.Width ;tloustka spiraly (ma vyznam polomeru)
v=point2-point1
vx=v(0) & vy=v(1) & vz=v(2) ;parametry primky
Ax=point1(0) & Ay=point1(1) & Az=point1(2)

;-------------------1. priblizeni--------------------------------
coeff=[Ax^2+Ay^2-(R+w)^2,2*Ax*vx+2*Ay*vy,vx^2+vy^2] ;pruseciky primky a valce obepinajiciho spiralu
inner_points=Get_inner_points(coeff,[vx,vy,vz,Ax,Ay,Az],limit=h)
p1=Get_plane_points([0.0,0.0,h],[vx,vy,vz,Ax,Ay,Az],limit=[R+w,0.0])
p2=Get_plane_points([0.0,0.0,0.0],[vx,vy,vz,Ax,Ay,Az],limit=[R+w,0.0])
if (n_elements(p1) eq 1) then if (n_elements(p2) eq 1) then plane_points=-1 else plane_points=p2 $
                         else if (n_elements(p2) eq 1) then plane_points=p1 else plane_points=[[p1],[p2]]
limit_points=Make_couple(plane_points,inner_points)
if n_elements(limit_points) ne 6 then return,-1 ;pokud nenaslo dva body

;------------------2. priblizeni---------------------------------
zd=limit_points(2,0)<limit_points(2,1) ;meze odkud kam prochazet spiralu
zu=limit_points(2,0)>limit_points(2,1)
td=2*!pi*zd/d
tu=2*!pi*zu/d
tlevels=findgen(levels)/(levels-1)*(tu-td)+td ;pocet intervalu pro rozdeleni spiraly
distance_array=dblarr(levels)
coord_array=dblarr(levels,3)

for i=0,levels-1 do begin ;pro kazdy level spocti prusecik primky s rovinou kolmou k vektru rychlosti
      S=[R*cos(tlevels(i)),R*sin(tlevels(i)),tlevels(i)*d/(2*!pi)] ;bod na spirale odpovidajici danemu t
      coeffs=Find_coeffs(tlevels(i),S,R,d,vx,vy,vz,Ax,Ay,Az)
      if (coeffs(0) eq 0.0) then begin    ;primka je rovnobezna s rovinou
         if (coeffs(1) eq 0.0) then begin ;primka lezi v rovine, spoctem pruseciky a koncime, jine pruseciky nejsou
            coeff=[Ax^2+Ay^2+Az^2+S(0)^2+S(1)^2+S(2)^2-2*Ax*S(0)-2*Ay*S(1)-2*Az*S(2)-w^2,$
                   2*(Ax*vx+Ay*vy+Az*vz-S(0)*vx-S(1)*vy-S(2)*vz),$
                   vx^2+vy^2+vz^2]    ;hledam pruseciky jako reseni rovnic: rovnice primky a rovnice koule
            t=fz_roots(coeff,/double) ;(x-S(0))^2+(y-S(1))^2+(z-S(2))^2=width^2
            if (imaginary(t(0)) ne 0.0) then return,-1 else begin ;pokud neni realne reseni (neni prusecik)
               t=double(t)
               x=Ax+t*vx ;souradnice pruseciku
               y=Ay+t*vy
               z=Az+t*vz
               return,transpose([[x],[y],[z],[double([tlevels(i),tlevels(i)])]])
            endelse
         endif else distance_array(i)=100.0 ;neexistuje zadny prusecik, jako vzdalenost dame velkou hodnotu
      endif else begin    ;primka neni rovnobezna s rovinou, spoctem vzdalenost pruseciku od bodu na spirale
         t=-coeffs(1)/coeffs(0)
         coord_array(i,0)=Ax+t*vx
         coord_array(i,1)=Ay+t*vy
         coord_array(i,2)=Az+t*vz
         distance_array(i)=sqrt((coord_array(i,0)-S(0))^2+(coord_array(i,1)-S(1))^2+(coord_array(i,2)-S(2))^2)-w
      endelse
endfor

;Mam pole se vzdalenostmi a budu hledat intervaly, ve kterych jsou pruseciky. Kde jsou 0.0,
;tam jsou pruseciky (count1), dalsi pruseciky jsou tam, kde prechazi znaminko (count2).
index_0=where(distance_array eq 0.0,count1) ;indexy na kterych je vzdalenost pruseciku od bodu spiraly w
distance_array_sgn=Signum(distance_array)  ;signum pole
distance_array_sft=shift(distance_array_sgn,-1) ;posunute signum o 1 doleva
distance_array_res=abs(distance_array_sgn-distance_array_sft)
index_2=where(distance_array_res(0:n_elements(distance_array_res)-2) eq 2.0,count2)
;Kde je v poli 2, tam je interval s prusecikem. Posunem se 2 muze dostat i na konec pole, to ale interval neznaci

;------------------3. priblizeni---------------------------------
if (count1+count2) ne 0 then begin ;pokud jsou nejake pruseciky
     coord_points=dblarr(count1+count2,4) ;pole pro ukladani pruseciku
     if count1 ne 0 then coord_points(0:count1-1,*)=[[coord_array(index_0,*)],tlevels(index_0)]
     if count2 ne 0 then begin  ;numericke hledani pruseciku
        for i=0,count2-1 do coord_points(i+count1,*)=Numerical_search([tlevels(index_2(i)),tlevels(index_2(i)+1)],$
                                                     [distance_array(index_2(i)),distance_array(index_2(i)+1)], $
                                                     [R,d,w],[vx,vy,vz,Ax,Ay,Az],1,accuracy)
     endif
     return,transpose(coord_points)
endif else return,-1
end

;------------------------- GET_POINTS ------------------------------
;pocita pruseciky primky s objekterm resenim kvadraticke rovnice
function Get_points,point1,point2,State
;rovnice primky x=Ax+t*vx, y=Ay+t*vy, z=Az+t*vz
;rovnice kuzele x^2+y^2-z^2*(R^2/h^2)=0
;rovnice polokoule x^2+y^2+(z-R)^2=R^2
;rovnice elipsoidu x^2/b^2+y^2/b^2+(z-a)^2/a^2=1 kde b^2=a^2*(1-e^2)
;rovnice valce   x^2+y^2=R^2
R=State.Radius
h=State.Height
a=State.a
b=sqrt(a^2*(1-State.exc^2))
v=point2-point1
vx=v(0) & vy=v(1) & vz=v(2)
Ax=point1(0) & Ay=point1(1) & Az=point1(2)
R_v=R-R*State.Distribution
b_v=b-b*State.Distribution

case State.ActualObject of
      'cone':begin
             border=h
             coeff=[Ax^2+Ay^2-R^2*Az^2/h^2,2*Ax*vx+2*Ay*vy-2*Az*vz*R^2/h^2,vx^2+vy^2-vz^2*R^2/h^2]
             coeff_v=[Ax^2+Ay^2-(R_v)^2*Az^2/h^2,2*Ax*vx+2*Ay*vy-2*Az*vz*(R_v)^2/h^2,vx^2+vy^2-vz^2*(R_v)^2/h^2]
             plane_points=Get_plane_points([0.0,0.0,h],[vx,vy,vz,Ax,Ay,Az],limit=[R,R_v])
             end
  'ellipsoid':begin
             border=a
             coeff=[Ax^2*a^2+Ay^2*a^2+Az^2*b^2-2*Az*a*b^2,$
                    2*Ax*vx*a^2+2*Ay*vy*a^2+2*Az*vz*b^2-2*vz*a*b^2,$
                    vx^2*a^2+vy^2*a^2+vz^2*b^2]
             plane_points=Get_plane_points([0.0,0.0,a],[vx,vy,vz,Ax,Ay,Az],limit=[b,b_v])
             coeff_v=[Ax^2*a^2+Ay^2*a^2+Az^2*b_v^2-2*Az*a*b_v^2,$
                         2*Ax*vx*a^2+2*Ay*vy*a^2+2*Az*vz*b_v^2-2*vz*a*b_v^2,$
                         vx^2*a^2+vy^2*a^2+vz^2*b_v^2]
             end
  'cylinder':begin
             border=h
             coeff=[Ax^2+Ay^2-R^2,2*Ax*vx+2*Ay*vy,vx^2+vy^2]
             coeff_v=[Ax^2+Ay^2-R_v^2,2*Ax*vx+2*Ay*vy,vx^2+vy^2]
             p1=Get_plane_points([0.0,0.0,h],[vx,vy,vz,Ax,Ay,Az],limit=[R,R_v])
             p2=Get_plane_points([0.0,0.0,0.0],[vx,vy,vz,Ax,Ay,Az],limit=[R,R_v])
             if (n_elements(p1) eq 1) then if (n_elements(p2) eq 1) then plane_points=-1 else plane_points=p2 $
                                      else if (n_elements(p2) eq 1) then plane_points=p1 else plane_points=[[p1],[p2]]
             end
endcase

inner_points=Get_inner_points(coeff,[vx,vy,vz,Ax,Ay,Az],limit=border) ;pruseciky s objektem
inner_points_v=Get_inner_points(coeff_v,[vx,vy,vz,Ax,Ay,Az],limit=border)

if (State.Distribution eq 0.0) then return,inner_points
if (State.Distribution eq 1.0) then return,Make_couple(plane_points,inner_points)

return,Make_couples(inner_points,inner_points_v,plane_points)
end

;------------------------- GENERATE ---------------------------------
pro Generate,State

if (State.SlitWid ne 0.01) or (State.ActualObject eq 'spiral') then begin ;waitbase
   widget_control,State.wTopBase,tlb_get_offset=offset
   waitbase=widget_base(xsize=230,ysize=70,tlb_frame_attr=1+8+2+4,/align_center,/row,$
                        xoffset=offset(0)+State.xTop*0.65,yoffset=offset(1)+State.yTop*0.5)
   waitlabel=widget_label(waitbase,value='       WAIT!!! GENERATING SPECTRA')
   widget_control,waitbase,/realize
   State.wWaitBase=waitbase
   widget_control,State.wParamBase,sensitive=0
   widget_control,State.wTopBase,sensitive=0
endif

over=200 ;o kolik ma byt spektrum delsi nez okno (aby to pri posouvani nelezlo na druhou stranu)
spectra=dblarr(State.xdim,State.ydim)
row=100*exp(-(dindgen(State.xdim+2*over)-(State.xdim+2*over)/2)^2/State.LineWidth) ;floating underflow
Matrix=State.Matrix(0:2,0:2) ;transformacni matice mezi soustavou okna a soustavou objektu
konst1=1.0 ;zmena vlnove delky = konst1 * dopplerovska rychlost
konst2=10000 ;posun ve spektru v pixlech = konst2 * zmena vlnove delky
;tyto konstanty jsou zvoleny tak, aby se zobrazovalo spektrum

for i=0,State.ydim-1 do begin ;pro kazdy bod sterbiny
step_number=(fix(State.SlitWid*100))<(State.Nstep)
for j=0,step_number-1 do begin
  point1=transpose([State.SlitPos+j*State.SlitWid/step_number,2.0*i/State.ydim-1.0,1.0]) # invert(Matrix)  ;body pro primku prepoctene
  point2=transpose([State.SlitPos+j*State.SlitWid/step_number,2.0*i/State.ydim-1.0,-1.0]) # invert(Matrix) ;do soustavy objektu

  if (State.ActualObject ne 'spiral') then points=Get_points(point1,point2,State) $
                                      else points=Get_spiral_points(point1,point2,State)
                                      ;vysledkem jsou pruseciky primky s objektem vyjadrene
                                      ;v soustave objektu (points=-1 pokud zadne nejsou)
     if n_elements(points) gt 1 then begin ;pokud existuje prusecik primky s objektem (points=-1 pokud neexistuje)

     ;--------------------------objem---------------------------------------------------------
     if (State.Distribution ne 0.0) then begin
        if (State.ActualObject ne 'spiral') then begin ;KUZEL, VALEC, ELIPSOID
             if ((n_elements(points)/3) mod 2) eq 0 then begin ;pokud sudy pocet
             for k=0,(n_elements(points)/6)-1 do begin
                 p1=points(*,k*2)
                 p2=points(*,k*2+1)
                 dist=sqrt((p1(0)-p2(0))^2+(p1(1)-p2(1))^2+(p1(2)-p2(2))^2)
                 dl=State.step/5.0
                 num=round(dist/dl)+1
                 if num eq 1 then num=2
                 v=p1-p2  ;vytvorime usecku x=Ax+vx*t,... t=od 0 do 1, tu rozdelime, spoctem
                 vx=v(0) & vy=v(1) & vz=v(2) ;spektrum pro dany bod a zintegrujem
                 Ax=p2(0) & Ay=p2(1) & Az=p2(2)
                 t=dindgen(num)/(num-1) ;pole parametru usecky, ktere urcuji body na usecce
                 ;pro kazdy bod zjistit rychlost, urcit radek a pak vsechny radky secist
                 for l=0,num-1 do begin
                     velocity=Get_velocity([Ax+t(l)*vx,Ay+t(l)*vy,Az+t(l)*vz],State)
                     deltalambda=konst1*Get_doppler(velocity,point1,point2)
                     lambdashift=round(konst2*deltalambda)
                     shiftrow=shift(row,-lambdashift)
                     spectra(*,i)=spectra(*,i)+shiftrow(over:over+State.xdim-1)
                 endfor
               endfor
              endif
         endif else begin ;SPIRALA
              if ((n_elements(points)/4) mod 2) eq 0 then begin ;pokud je sudy pocet pruseciku
                 for k=0,(n_elements(points)/8)-1 do begin ;pro kazdou dvojici bodu
                     p1=points(*,k*2)
                     p2=points(*,k*2+1)
                     dist=sqrt((p1(0)-p2(0))^2+(p1(1)-p2(1))^2+(p1(2)-p2(2))^2)
                     dl=State.step/5.0
                     num=round(dist/dl)+1
                     if num eq 1 then num=2
                     t=findgen(num)/(num-1)*(p2(3)-p1(3))+p1(3)
                     for l=0,num-1 do begin
                         velocity=Get_spiral_velocity(t(l),State)
                         deltalambda=konst1*Get_doppler(velocity,point1,point2)
                         lambdashift=round(konst2*deltalambda)
                         shiftrow=shift(row,-lambdashift)
                         spectra(*,i)=spectra(*,i)+shiftrow(over:over+State.xdim-1)
                     endfor
                 endfor
              endif
         endelse
     ;-------------------------povrch------------------------------------------------------------
     endif else begin
                    s=size(points)
                    for k=0,(n_elements(points)/s(1))-1 do begin ;pro kazdy prusecik
                        if (State.ActualObject ne 'spiral') then velocity=Get_velocity(points(*,k),State) $ ;vektor rychlosti
                                                            else velocity=Get_spiral_velocity(points(3,k),State)
                        deltalambda=konst1*Get_doppler(velocity,point1,point2)  ;zmenu vlnove delky
                        lambdashift=round(konst2*deltalambda) ;posun ve spektru v pixlech
                        shiftrow=shift(row,-lambdashift)
                        spectra(*,i)=spectra(*,i)+shiftrow(over:over+State.xdim-1)
                    endfor
     endelse
     endif
endfor
endfor

widget_control,State.wRightDraw,get_value=window1 ;zobrazeni spektra
wset,window1
if (State.Negativ eq 0) then tv,bytscl(spectra) else tv,255-bytscl(spectra)
State.spectra=spectra

redraw,State.xdim,State.ydim,State.wRightDraw,condition=State.wToggleButton1 ;vykresleni stredove cary pokud existovala

if (State.SlitWid ne 0.01) or (State.ActualObject eq 'spiral') then begin ;zrusit waitbase pokud existuje
   if widget_info(State.wWaitBase,/valid_id) then widget_control,State.wWaitBase,/destroy
   widget_control,State.wTopBase,sensitive=1
   widget_control,State.wParamBase,sensitive=1
endif

widget_control,State.wTopBase,set_uvalue=State

end
