FUNCTION cone_bottom::Init, POS=pos, RADIUS=radius, INNER=inner, HEIGHT=height, DENSITY=density, $
                     _EXTRA=e
    IF (self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0
    self.pos = [0.0,0.0,0.0]
    self.radius = 0.7
    self.height= 0.7
    self.density = 10
    self.inner=0.0
    IF (N_ELEMENTS(pos) EQ 3) THEN self.pos = pos
    IF (N_ELEMENTS(radius) EQ 1) THEN self.radius = radius
    IF (N_ELEMENTS(height) EQ 1) THEN self.height = height
    IF (N_ELEMENTS(density) EQ 1) THEN self.density = density
    IF (N_ELEMENTS(inner) EQ 1) THEN self.inner = inner
    self.oPoly = OBJ_NEW('IDLgrPolygon', SHADING=1, _EXTRA=e)
    self->Add,self.oPoly
    self->BuildPoly
    RETURN, 1
END

;----------------------------------------------------------------------------
PRO cone_bottom::Cleanup
    OBJ_DESTROY, self.oPoly
    self->IDLgrModel::Cleanup
END

;----------------------------------------------------------------------------
PRO cone_bottom::SetProperty, POS=pos, RADIUS=radius, INNER=inner, HEIGHT=height, DENSITY=density, _EXTRA=e
    self->IDLgrModel::SetProperty, _EXTRA=e
    self.oPoly->SetProperty, _EXTRA=e
    IF (N_ELEMENTS(pos) EQ 3) THEN self.pos = pos
    IF (N_ELEMENTS(radius) EQ 1) THEN self.radius = radius
    IF (N_ELEMENTS(inner) EQ 1) THEN self.inner = inner
    IF (N_ELEMENTS(height) EQ 1) THEN self.height = height
    IF (N_ELEMENTS(density) EQ 1) THEN self.density = density
    self->BuildPoly
END

;----------------------------------------------------------------------------
PRO cone_bottom::GetProperty, POS=pos, RADIUS=radius, INNER=inner, HEIGHT=height, DENSITY=density,$
                         POBJ=pobj, _EXTRA=e
    self->IDLgrModel::GetProperty, _EXTRA=e
    self.oPoly->GetProperty, _EXTRA=e
    pos = self.pos
    radius = self.radius
    inner=self.inner
    height= self.height
    density = self.density
    pobj = self.oPoly
END

;----------------------------------------------------------------------------
PRO cone_bottom::BuildPoly

    ;---------------data-----------------
    radius2=self.radius-self.radius*self.inner
    nrad=fix(self.density*self.radius*self.inner/sqrt((self.radius*self.inner)^2+(self.height)^2))
    if (nrad lt 3) then nrad=3
    rad=(self.radius-radius2)*findgen(nrad)/(nrad-1)+radius2
    fi=2*!pi*findgen(4*(self.density-1))/(4*(self.density-1))
    nfi=n_elements(fi)
    n=nrad*nfi

    x=findgen(n)
    y=findgen(n)
    z=findgen(n)

    for i=0,nrad-1 do begin
        i2=nrad-1-i
        for j=0,nfi-1 do begin
            x(j+i*nfi)=rad(i2)*cos(fi(j))+self.pos(0)
            y(j+i*nfi)=rad(i2)*sin(fi(j))+self.pos(1)
            z(j+i*nfi)=self.height+self.pos(2)
        endfor
    endfor

    data=transpose([[x],[y],[z]])

    ;--------------polygons----------------------------

    dim=nfi*(nrad-1)
    polygon1=intarr(5,dim)
    polygon1(0,*)=4
    polygon1(1,*)=indgen(dim)
    polygon1(2,*)=indgen(dim)+1
    polygon1(3,*)=indgen(dim)+nfi+1
    polygon1(4,*)=indgen(dim)+nfi
    for i=0,nrad-2 do begin
        polygon1(2,(i+1)*nfi-1)=polygon1(1,i*nfi)
        polygon1(3,(i+1)*nfi-1)=polygon1(4,i*nfi)
    endfor
    polygon1=reform(polygon1,5*dim)

    self.oPoly->SetProperty, DATA=data , POLYGONS=polygon1

END

;-----------------------------------------------------------
PRO cone_bottom_define
    struct = { cone_bottom, $
               INHERITS IDLgrModel, $
               pos: [0.0,0.0,0.0], $
               radius: 0.7, $
               inner: 0.0, $
               height: 0.7, $
               density: 10, $
               oPoly: OBJ_NEW() $
             }
END
