FUNCTION cone::Init, POS=pos, RADIUS=radius, HEIGHT=height, DENSITY=density, $
                     _EXTRA=e
    IF (self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0
    self.pos = [0.0,0.0,0.0]
    self.radius = 0.7
    self.height= 0.7
    self.density = 10
    IF (N_ELEMENTS(pos) EQ 3) THEN self.pos = pos
    IF (N_ELEMENTS(radius) EQ 1) THEN self.radius = radius
    IF (N_ELEMENTS(height) EQ 1) THEN self.height = height
    IF (N_ELEMENTS(density) EQ 1) THEN self.density = density
    self.oPoly = OBJ_NEW('IDLgrPolygon', SHADING=1, _EXTRA=e)
    self->Add,self.oPoly
    self->BuildPoly
    RETURN, 1
END

;----------------------------------------------------------------------------
PRO cone::Cleanup
    OBJ_DESTROY, self.oPoly
    self->IDLgrModel::Cleanup
END

;----------------------------------------------------------------------------
PRO cone::SetProperty, POS=pos, RADIUS=radius, HEIGHT=height, DENSITY=density, _EXTRA=e
    self->IDLgrModel::SetProperty, _EXTRA=e
    self.oPoly->SetProperty, _EXTRA=e
    IF (N_ELEMENTS(pos) EQ 3) THEN self.pos = pos
    IF (N_ELEMENTS(radius) EQ 1) THEN self.radius = radius
    IF (N_ELEMENTS(height) EQ 1) THEN self.height = height
    IF (N_ELEMENTS(density) EQ 1) THEN self.density = density
    self->BuildPoly
END

;----------------------------------------------------------------------------
PRO cone::GetProperty, POS=pos, RADIUS=radius, HEIGHT=height, DENSITY=density,$
                         POBJ=pobj, _EXTRA=e
    self->IDLgrModel::GetProperty, _EXTRA=e
    self.oPoly->GetProperty, _EXTRA=e
    pos = self.pos
    radius = self.radius
    height= self.height
    density = self.density
    pobj = self.oPoly
END

;----------------------------------------------------------------------------
PRO cone::BuildPoly

    ;---------------data-----------------
    distance=self.height*findgen(self.density)/(self.density-1)
    fi=2*!pi*findgen(4*(self.density-1))/(4*(self.density-1))
    nfi=n_elements(fi)
    ndistance=n_elements(distance)
    n=(ndistance-1)*nfi+1

    x=findgen(n)
    y=findgen(n)
    z=findgen(n)

    for i=0,ndistance-2 do begin
        i2=ndistance-1-i
        for j=0,nfi-1 do begin
            x(j+i*nfi)=(distance(i2)*self.radius/self.height)*cos(fi(j))+self.pos(0)
            y(j+i*nfi)=(distance(i2)*self.radius/self.height)*sin(fi(j))+self.pos(1)
            z(j+i*nfi)=distance(i2)+self.pos(2)
        endfor
    endfor
    x(n-1)=0.0+self.pos(0)
    y(n-1)=0.0+self.pos(1)
    z(n-1)=0.0+self.pos(2)

    data=transpose([[x],[y],[z]])

    ;--------------polygons----------------------------

    dim=nfi*(ndistance-2)
    polygon1=intarr(5,dim)
    polygon1(0,*)=4
    polygon1(1,*)=indgen(dim)
    polygon1(2,*)=indgen(dim)+1
    polygon1(3,*)=indgen(dim)+nfi+1
    polygon1(4,*)=indgen(dim)+nfi
    for i=0,ndistance-3 do begin
        polygon1(2,(i+1)*nfi-1)=polygon1(1,i*nfi)
        polygon1(3,(i+1)*nfi-1)=polygon1(4,i*nfi)
    endfor
    polygon1=reform(polygon1,5*dim)

    polygon2=intarr(4,nfi)
    polygon2(0,*)=3
    polygon2(1,*)=indgen(nfi)+dim
    polygon2(2,*)=indgen(nfi)+dim+1
    polygon2(3,*)=n-1
    polygon2(2,nfi-1)=polygon2(1,0)
    polygon2=reform(polygon2,4*nfi)

    polygons=[polygon1,polygon2]

    self.oPoly->SetProperty, DATA=data , POLYGONS=polygons

END

;-----------------------------------------------------------
PRO cone_define
    struct = { cone, $
               INHERITS IDLgrModel, $
               pos: [0.0,0.0,0.0], $
               radius: 0.7, $
               height: 0.7, $
               density: 10, $
               oPoly: OBJ_NEW() $
             }
END







